subroutine linear_wave

!--------------------------------------------------------------
!SAM specific routine parameterized large-scale linear wave dynamics
! solving 2D qgw equation, see Nie and Sobel 2016, Nie, Shaevitz, and Sobel 2016 for details. 
!--------------------------------------------------------------


!------------------------------------- free parameters
! wavelength: horizontal wave length
! ztop:   upper boundary of the qgw equation
! zlow:   lower boundary of the qgw equation
!------------------------------------- external inputs
! advt, advr, advzeta: horizontal advection of temperautre, moisture, and absolute vorticity. Their dimension should be (time, nlevel)
! w_bc: w at lower boundary. Its dimension should be (time)
!------------------------------------- inputs from CRM internal variables
! dt_conv: the sum of convective and radiative heating, obtained from CRM simulation at every time step

use vars
use params
use module_linearwave
use microphysics, only: micro_field, index_water_vapor, total_water
implicit none
integer k,i,j,nn

real dwwavedt(nzm)
real wn   !horizontal wavenumber
real ztop     !height of the rigid lid if the regid lid is used
integer kztop !height index of the rigid lid
real zlow     !in qgw, is the lower boundary layer height
integer kzlow !height index of the lower rigid lid
real sigma_T(nzm), F_zero(nzm)

real coef

!the height of the rigid lid
ztop=12500. ! (leve=l32)
zlow=50.
!tendency of large-scale wave vertical velocity
dwwavedt=0.


t_wavebg=tabs0
q_wavebg=qv0
F_zero=0.

!----------------------------------------
if (nstep.eq.nstartlinearwave+nsteplinearwavebg) then
if (masterproc) print *, 'linear wave is on'
endif

!Start wave coupling after the reference profiles have been obtained.
if(nstep.gt.nstartlinearwave+nsteplinearwavebg) then 
   !accumulate T, q over nsteplinearwave
   t_wave_local=t_wave_local+tabs0
   q_wave_local=q_wave_local+qv0
   !jinie: more averaging
   ! convective+radiative heating+lsf heating+res  K/s
   ! if lsf heating is constant in time, it will not generate QGw in equilibrium
   dt_conv_local=dt_conv_local+(tlat(1:nzm)+tlatqi(1:nzm)+tadv(1:nzm)+tdiff(1:nzm))/float(nx*ny)/dtn+(radqrlw(1:nzm)+radqrsw(1:nzm))/float(nx*ny)+ttend(1:nzm)-res_qgw(1:nzm)

!update w every nsteplinearwave steps
   if(mod(nstep-nstartlinearwave-nsteplinearwavebg,nsteplinearwave).eq.0) then

if (masterproc) print *, 'calculate linear wave'

      !average T, q over nsteplinearwave
      t_wave=t_wave_local/float(nsteplinearwave)
      q_wave=q_wave_local/float(nsteplinearwave)
      ! jinie: more averaging 
      dt_conv=dt_conv_local/float(nsteplinearwave)
  
      !compute virtual temperature
      tv_wavebg=t_wavebg*(1+0.61*q_wavebg)
      tv_wave=t_wave*(1+0.61*q_wave)

	! wn is horizontal wave number (i.e., k in equations in Nie and Sobel 2016)
         wn= 6.283e-3/wavelength    
 
         !rigid lid upper boundary condition at ztop
	! kzlow is the lowest level where the qgw equation is solved
	! kztop is the highest level where the qgw equation is solved
         kzlow=1    
         do k=1,nzm-1
            if(z(k).le.ztop.and.z(k+1).gt.ztop) kztop=k
            if(z(k).le.zlow.and.z(k+1).gt.zlow) kzlow=k
         end do
        
	! to calculate dry static stability
	! t_wavebg is the CRM domain averaged T profiles averaged over some time period
       do k=2,nzm-1 
          sigma_T(k)= (t_wavebg(k+1)-t_wavebg(k-1))/(z(k+1)-z(k-1))+ggr/Cp
       enddo
          sigma_T(1)= (t_wavebg(2)-t_wavebg(1))/(z(2)-z(1))+ggr/Cp

        !-------time varying advective forcing
        do i=1,nt_forcing
           if( (day0+time/86400).gt.time_forcing(i)) then
             nn=i
           endif
        end do

        if (nn==nt_forcing) then
           if(masterproc) print*, 'forcing period is too short.'
           stop
        endif
                coef=(day0+time/86400-time_forcing(nn))/(time_forcing(nn+1)-time_forcing(nn))

        if ((coef<0).or.(coef>1)) then
           if(masterproc) print*, 'forcing period is too short.'
           stop
        endif

                do i=1,nzm
                advt(i)=advt_all(nn,i)+(advt_all(nn+1,i)-advt_all(nn,i))*coef
                advr(i)=advr_all(nn,i)+(advr_all(nn+1,i)-advr_all(nn,i))*coef
                advzeta(i)=advzeta_all(nn,i)+(advzeta_all(nn+1,i)-advzeta_all(nn,i))*coef
                enddo
                w_bc=w_bc_all(nn)+(w_bc_all(nn+1)-w_bc_all(nn))*coef

! -------calculate f_zeta, f_t, f_ftnet
! F_Q = - \frac{g}{\bar T} k^2 /f^2 Q , where Q is the sum of convective and radiative heating
    do i=2,nzm-1
      F_zeta(i)=-(advzeta(i+1)-advzeta(i-1))/(z(i+1)-z(i-1))/ff0              
      F_t(i)=-ggr/t_wavebg(i)*wn*wn/ff0/ff0*advt(i)
! add sort of damping
      F_Q(i)=-ggr/t_wavebg(i)*wn*wn/ff0/ff0*dt_conv(i)*0.98
    end do
    F_zeta(1)=-(advzeta(2)-advzeta(1))/(z(2)-z(1))/ff0
    F_t(1)=-ggr/t_wavebg(1)*wn*wn/ff0/ff0*advt(1)
    F_Q(i)=-ggr/t_wavebg(1)*wn*wn/ff0/ff0*dt_conv(1)
    F_zeta(nzm)=0
    F_t(nzm)=0
    F_Q(nzm)=0

! test cases, turn off Q feedback
!        F_Q=0.

!   solve qgw equation
         call calc_qgw(wn,kztop,kzlow,ff0,z(1:kztop),zi(1:kztop+1),rho(1:kztop),sigma_T(1:kztop),t_wavebg(1:kztop),F_zeta(1:kztop)+F_t(1:kztop)+F_Q(1:kztop),w_bc,w_wave(1:kztop))

         call calc_qgw(wn,kztop,kzlow,ff0,z(1:kztop),zi(1:kztop+1),rho(1:kztop),sigma_T(1:kztop),t_wavebg(1:kztop),F_zeta(1:kztop),0.,w_wave_advzeta(1:kztop))

         call calc_qgw(wn,kztop,kzlow,ff0,z(1:kztop),zi(1:kztop+1),rho(1:kztop),sigma_T(1:kztop),t_wavebg(1:kztop),F_t(1:kztop),0.,w_wave_advT(1:kztop))

         call calc_qgw(wn,kztop,kzlow,ff0,z(1:kztop),zi(1:kztop+1),rho(1:kztop),sigma_T(1:kztop),t_wavebg(1:kztop),F_Q(1:kztop),0.,w_wave_Q(1:kztop))

         call calc_qgw(wn,kztop,kzlow,ff0,z(1:kztop),zi(1:kztop+1),rho(1:kztop),sigma_T(1:kztop),t_wavebg(1:kztop),F_zero(1:kztop),w_bc,w_wave_bc(1:kztop))

      end if    

      t_wave_local=0.
      q_wave_local=0.
      dt_conv_local=0.
      
end if

end subroutine linear_wave

