!---------------------------------------------
!Two subroutines that performs the coupling between a single column model 
!(or a limited domain cloud resolving model)
!and a 2D gravity wave with a single horizontal wave number
!Reference: Kuang ,2008, Journal of Atmospheric Sciences, pg 576-591
! Nie and Sobel JAS, 2016
!Zhiming Kuang 2008, and Ji Nie 2018
! 
!---------------------------------------------
!There are 3 subroutines
!1. calc_wtend computes the tendency for vertical velocity, given the current virtual temperature profile.
!2. linearwavesubsidence computes the temperature and moisture tendencies given the current vertical velocity profile 
!3. calculate w using qgw equation, Nie and Sobel 16

module module_linearwave

  use grid, only : masterproc

  implicit none
  private
  real, parameter:: ggr=9.8  !gravity m/s2
  real, parameter:: Cp=1004. !specific heat J/kg/K
  public :: calc_wtend, linearwavesubsidence, linearwavesubsidence2, calc_qgw
  
contains

!------------------------------------------------------------------------------
!Compare the current sounding with the reference sounding, from which infer the vertical velocity tendency
      subroutine calc_wtend(           wn,  w_curr, & 
      tv_curr,        tv_fullbg,       rho_full ,   &
      z_full,                          z_half,      &
      wtend,                           nz, &
      rigid_upper_bc)

!     ------------------------------ input arguments ------------------------------

      integer, intent(in) ::                                                    &
      nz                        !number of midpoint levels, the number of interface levels is nz+1

      real, dimension(nz), intent(in) ::                 &
      tv_curr,           &       ! Cell center virtual temperature
      w_curr,           &       ! Cell center wave vertical velocity
      tv_fullbg,          &       ! Cell center background virtual temperature
      z_full,           &       ! Cell center height
      rho_full                  ! Cell center density
      real, dimension(nz+1), intent(in) ::                 &
      z_half                    ! Interface-level height, including the surface, which is at height zero

      real, intent (in) ::       wn        !the specified horizontal wavenumber

      logical, intent(in) ::     rigid_upper_bc   !If .false., a radiation upper BC will be used
!     ------------------------------ output arguments -----------------------------

      real, dimension(nz), intent(out) ::                 &
      wtend                  ! Cell center w tendency

!     ------------------------------ local variables ------------------------------
      real  N2top !Top-level buoyancy frequency squared

      real, dimension(1:nz+1) ::                                             &
      dz                   ! grid spacing between midpoint levels
      real, dimension(1:nz) ::                                             &
      rhs,                 &  ! The right hand side for the Gauss Elimination
      aa,                 &  !aa, bb, cc are the three columns of the tridiagonal matrix 
      bb,                 &  
      cc                   

      integer ::                                                                &
      k                         ! Vertical loop counter

!     ------------------------------ executable code ------------------------------

      N2top=ggr/tv_fullbg(nz)*((tv_fullbg(nz)-tv_fullbg(nz-1))/(z_full(nz)-z_full(nz-1))+ggr/Cp)

!     compute grid spacing between midpoint levels
      do k=2,nz
         dz(k)=z_full(k)-z_full(k-1)
      enddo
      dz(1)=2*z_full(1)
      dz(nz+1)=2*(z_half(nz+1)-z_full(nz))
      
!     Gauss Elimination
      rhs=0.
      do k=1,nz 
         rhs(k)=-rho_full(k)*ggr*wn*wn*(tv_curr(k)-tv_fullbg(k))/tv_fullbg(k)*dz(k)*dz(k+1)/2.
      enddo
      
!     set up the tridiagonal matrix
      do k=1,nz
         aa(k)=dz(k+1)/(dz(k)+dz(k+1))
         bb(k)=-1.
         cc(k)=dz(k)/(dz(k)+dz(k+1))
      enddo
      
      !symmetric lower BC
      aa(1)=0.
      bb(1)=-(2*dz(2)+dz(1))/(dz(1)+dz(2))

      !Upper BC
      if(rigid_upper_bc) then         
                                !symmetric upper BC
         cc(nz)=0.
         bb(nz)=-(2*dz(nz)+dz(nz+1))/(dz(nz)+dz(nz+1))
      else
                                !Radiating upper BC
         aa(nz)=dz(nz+1)/dz(nz)
         bb(nz)=-1.*dz(nz+1)/dz(nz)
         rhs(nz)=rhs(nz)+rho_full(nz)*sqrt(N2top)*wn*w_curr(nz)*dz(nz+1)
      endif
      
      !Gaussian Elimination with no pivoting
      do k=1,nz-1
         bb(k+1)=bb(k+1)-aa(k+1)/bb(k)*cc(k)
         rhs(k+1)=rhs(k+1)-aa(k+1)/bb(k)*rhs(k)
      enddo
      
      !backward substitution
      rhs(nz)=rhs(nz)/bb(nz)
      do k= nz-1,1,-1
         rhs(k)=(rhs(k)-cc(k)*rhs(k+1))/bb(k)
      enddo

      do k=1,nz
         wtend(k)=rhs(k)/rho_full(k)
      enddo


      end subroutine calc_wtend
!------------------------------------------------------------------------------
!Centered difference      
      subroutine linearwavesubsidence2(           w_curr,      &
      t_fullbg,                       q_fullbg,    &
      z_full,                          ttend,        &
      qtend,                           nz )

!     ------------------------------ input arguments ------------------------------

      integer, intent(in) ::                                                    &
      nz                        !number of midpoint levels, the number of interface levels is nz+1

      real, dimension(nz), intent(in) ::                 &
      t_fullbg,           &       ! Cell center background temperature
      q_fullbg,           &       ! Cell center background moisture
      w_curr,          &       ! Cell center wave vertical velocity
      z_full                 ! Cell center height


!     ------------------------------ output arguments -----------------------------

      real, dimension(nz), intent(out) ::                 &
      ttend,             &     ! Cell center T tendency
      qtend                  ! Cell center q tendency

!     ------------------------------ local variables ------------------------------
      real ggr_over_Cp ! gravity divided specific heat

      integer ::                                                                &
      k                         ! Vertical loop counter

!     ------------------------------ executable code ------------------------------

      ggr_over_Cp=ggr/Cp

      do k=2,nz-1
         ttend(k) =  - w_curr(k) * ((t_fullbg(k+1)-t_fullbg(k-1))/(z_full(k+1)-z_full(k-1))+ggr_over_Cp)
         qtend(k) =  - w_curr(k) * (q_fullbg(k+1)-q_fullbg(k-1))/(z_full(k+1)-z_full(k-1))
      enddo

      ! Use 1st order formula at upper and lower boundaries
      ttend(1)=-w_curr(1)*((t_fullbg(2)-t_fullbg(1))/(z_full(2)-z_full(1))+ggr_over_Cp)
      qtend(1)=-w_curr(1)*(q_fullbg(2)-q_fullbg(1))/(z_full(2)-z_full(1))
      ttend(nz)=-w_curr(nz)*((t_fullbg(nz)-t_fullbg(nz-1))/(z_full(nz)-z_full(nz-1))+ggr_over_Cp)
      qtend(nz)=-w_curr(nz)*(q_fullbg(nz)-q_fullbg(nz-1))/(z_full(nz)-z_full(nz-1))

      end subroutine linearwavesubsidence2

!------------------------------------------------------------------------------
!upwind
      subroutine linearwavesubsidence(           w_curr,      &
      t_fullbg,                       q_fullbg,    &
      z_full,                          ttend,        &
      qtend,                           nz )

!     ------------------------------ input arguments ------------------------------

      integer, intent(in) ::                                                    &
      nz                        !number of midpoint levels, the number of interface levels is nz+1

      real, dimension(nz), intent(in) ::                 &
      t_fullbg,           &       ! Cell center background temperature
      q_fullbg,           &       ! Cell center background moisture
      w_curr,          &       ! Cell center wave vertical velocity
      z_full                 ! Cell center height


!     ------------------------------ output arguments -----------------------------

      real, dimension(nz), intent(out) ::                 &
      ttend,             &     ! Cell center T tendency
      qtend                  ! Cell center q tendency

!     ------------------------------ local variables ------------------------------
      real ggr_over_Cp ! gravity divided specific heat

      integer ::                                                                &
      k                         ! Vertical loop counter

!     ------------------------------ executable code ------------------------------

      ggr_over_Cp=ggr/Cp

      do k=2,nz-1
         if(w_curr(k)>0) then
            ttend(k) =  - w_curr(k) * ((t_fullbg(k)-t_fullbg(k-1))/(z_full(k)-z_full(k-1))+ggr_over_Cp)
            qtend(k) =  - w_curr(k) * (q_fullbg(k)-q_fullbg(k-1))/(z_full(k)-z_full(k-1))
         else
            ttend(k) =  - w_curr(k) * ((t_fullbg(k+1)-t_fullbg(k))/(z_full(k+1)-z_full(k))+ggr_over_Cp)
            qtend(k) =  - w_curr(k) * (q_fullbg(k+1)-q_fullbg(k))/(z_full(k+1)-z_full(k))
         end if
      end do

      !  upper and lower boundaries
      ttend(1)=-w_curr(1)*((t_fullbg(2)-t_fullbg(1))/(z_full(2)-z_full(1))+ggr_over_Cp)
      qtend(1)=-w_curr(1)*(q_fullbg(2)-q_fullbg(1))/(z_full(2)-z_full(1))
      ttend(nz)=-w_curr(nz)*((t_fullbg(nz)-t_fullbg(nz-1))/(z_full(nz)-z_full(nz-1))+ggr_over_Cp)
      qtend(nz)=-w_curr(nz)*(q_fullbg(nz)-q_fullbg(nz-1))/(z_full(nz)-z_full(nz-1))


      end subroutine linearwavesubsidence

!------------------------------------------------------------------------------
!solve qgw eq., see "numericical.pdf" for equations.
      subroutine calc_qgw(wn,nz,nlow,f0,z_full,z_half,rho_full,sigma_T,T_bg,F,w_bc,w)

!     ------------------------------ input arguments ------------------------------
      integer, intent(in) ::  nz      !number of midpoint levels, the number of interface levels is nz+1
      integer, intent(in) ::  nlow    !number of lower bc level
      real, intent (in) ::    wn,  &  !the specified horizontal wavenumber
                f0          ! coriolis param. only used in cal_qgw

      real, dimension(nz), intent(in) ::                 &
      z_full,          &       ! Cell center height
      rho_full,       &        ! Cell center density
      sigma_T,       &        ! Cell center \partial T \partial z + g/cp
      T_bg,       &        ! Cell center background temp.		
      F                    ! RHS of qgw eq. 
      real, intent(in) :: w_bc   ! lower b.c. for w: m/s

      real, dimension(nz+1), intent(in) ::                 &
      z_half                    ! Interface-level height, including the surface, which is at height zero

!     ------------------------------ output arguments -----------------------------
      real, dimension(nz), intent(out) ::  w                

!     ------------------------------ local variables ------------------------------
      real, dimension(1:nz+1) ::                                             &
      dz , &                  ! grid spacing between midpoint levels
      rho              ! Cell center density, with extra level
      real, dimension(1:nz) ::                                             &
      rhs,                &  ! The right hand side for the Gauss Elimination
      aa,                 &  !aa, bb, cc are the three columns of the tridiagonal matrix 
      bb,                 &  
      cc                   

      
      integer ::                                                                &
      k                         ! Vertical loop counter
      real :: rhop, rhon,  A, B2

!     ------------------------------ executable code ------------------------------

      w=0.

      rho(1:nz)=rho_full
      rho(nz+1)=rho(nz)

!     compute grid spacing between midpoint levels
      do k=2,nz
         dz(k)=z_full(k)-z_full(k-1)
      enddo
      dz(1)=dz(2)
      dz(nz+1)=dz(nz)
      
!     set up the tridiagonal matrix
      rhs=0.
      aa=0.
      bb=0.
      cc=0.
      do k=nlow,nz
        rhop=(rho(k)+rho(k+1))/2
        rhon=(rho(k)+rho(k-1))/2
        A=2./(dz(k)+dz(k+1))/(rhop*dz(k+1)*rhon*dz(k))
        B2=ggr*sigma_T(k)/rho(k)/T_bg(k)*wn*wn/f0/f0
        rhs(k)=F(k)
        aa(k)=A*rhop*dz(k+1);
        cc(k)=A*rhon*dz(k);
        bb(k)=-A*(rhop*dz(k+1)+rhon*dz(k))-B2
      enddo
      
!    at lowest model layer, w=w_bc*rho
      aa(nlow)=0.   
      bb(nlow)=1 
      cc(nlow)=0.
      rhs(nlow)=w_bc*rho(nlow)
!    at tropopause, w=0
      aa(nz)=0.
      bb(nz)=1.
      cc(nz)=0.
      rhs(nz)=0.
           
      !Gaussian Elimination with no pivoting
      do k=nlow,nz-1
         bb(k+1)=bb(k+1)-aa(k+1)/bb(k)*cc(k)
         rhs(k+1)=rhs(k+1)-aa(k+1)/bb(k)*rhs(k)
      enddo   
      !backward substitution
      rhs(nz)=rhs(nz)/bb(nz)
      do k= nz-1,nlow,-1
         rhs(k)=(rhs(k)-cc(k)*rhs(k+1))/bb(k)
      enddo

      do k=nlow,nz
         w(k)=rhs(k)/rho(k)
      enddo
      do k=1,nlow-1
         w(k)= w(nlow)/(z_full(1)-z_full(nlow))*(z_full(1)-z_full(k))
      enddo

      end subroutine calc_qgw




    end module module_linearwave
